#pragma once

#include <cstdint>
#include <iterator>

/// @brief Output iterator that only counts the number of outputs.
struct Counter
{
  using difference_type = std::ptrdiff_t;

  struct NoOpReference {
    template <typename T>
    auto operator=(T&&) const -> void {}
  };

  std::size_t n = 0;

  Counter() = default;
  constexpr Counter(std::size_t n) : n{n} {}

  // prefix increment
  auto operator++() -> Counter&
  {
    n++;
    return *this;
  }

  // postfix increment
  auto operator++(int) -> Counter
  {
    return n++;
  }

  auto operator*() const -> NoOpReference
  {
    return {};
  }

  auto operator-(Counter rhs) const -> difference_type
  {
    return static_cast<difference_type>(n) - static_cast<difference_type>(rhs.n);
  }

  operator std::size_t() const
  {
    return n;
  }
};

static_assert(std::output_iterator<Counter, int>);
